﻿using System;
using System.Runtime.InteropServices;
using System.Windows;
using Microsoft.Win32;
using System.Collections.Generic;
using System.Globalization;

namespace CSharp_LibWrapper
{
    #region XslHal class
    /// <summary>    
    /// Wrapper class for the XSL HAL DLL.
    /// </summary>
    /// <remarks>
    /// This class exposes wrapper functions for the XSL HAL DLL interface. By this interface, it is 
    /// possible to access several hardware related functions in an easy way.
    /// </remarks>
    /// 

    public enum DHCOM_GPIOEnum
    {
        DHCOM_GPIO_A = 1,
        DHCOM_GPIO_B,
        DHCOM_GPIO_C,
        DHCOM_GPIO_D,
        DHCOM_GPIO_E,
        DHCOM_GPIO_F,
        DHCOM_GPIO_G,
        DHCOM_GPIO_H,
        DHCOM_GPIO_I,
        DHCOM_GPIO_J,
        DHCOM_GPIO_K,
        DHCOM_GPIO_L,
        DHCOM_GPIO_M,
        DHCOM_GPIO_N,
        DHCOM_GPIO_O,
        DHCOM_GPIO_P,
        DHCOM_GPIO_Q,
        DHCOM_GPIO_R,
        DHCOM_GPIO_S,
        DHCOM_GPIO_T,
        DHCOM_GPIO_U,
        DHCOM_GPIO_V,
        DHCOM_GPIO_W,
        DHCOM_GPIO_UART1_CTS,
        DHCOM_GPIO_UART1_RTS,
        DHCOM_GPIO_UART2_CTS,
        DHCOM_GPIO_UART2_RTS,
        DHCOM_GPIO_UART3_CTS,
        DHCOM_GPIO_UART3_RTS,
        DHCOM_GPIO_UART4_CTS,
        DHCOM_GPIO_UART4_RTS,
        DHCOM_GPIO_UART5_CTS,
        DHCOM_GPIO_UART5_RTS,
        DHCOM_GPIO_NOT_DEFINED
    };

    public enum GPIOConvertType
    {
        DHCOM_GPIO_ID = 0,
        DHCOM_GPIO_PORT,
        DHCOM_GPIO_PIN
    };

    public enum DHCOM_I2CEnum
    {
        DHCOM_I2C0 = 0,
        DHCOM_I2C1,
        DHCOM_I2C2,
        DHCOM_I2C3,
        /*DHCOM_I2C4,*/
        DHCOM_I2C_NOT_DEFINED
    };

    public enum DHCOM_I2CFREQMode
    {
        I2C_100K = 0,
        I2C_400K,
        I2C_800K,
        I2C_1600K,
        I2C_2400K,
        I2C_3200K
    };

    public static class DHHalLib
    {
        #region P/Invoke declarations for VITA HAL DLL interface

        const string LibName = "DHHalLib.dll";

        // Version
        [DllImport(LibName, EntryPoint = "DHHalLibGetVersion")]
        public static extern UInt32 DHHalLibGetVersion();

        [DllImport(LibName, EntryPoint = "WinCEImageGetVersion")]
        public static extern UInt32 WinCEImageGetVersion();

        [DllImport(LibName, EntryPoint = "BSPGetVersion")]
        public static extern UInt32 BSPGetVersion();

        // GPIO
        [DllImport(LibName, EntryPoint = "GPIOSetDirection")]
        public static extern bool GPIOSetDirection(DHCOM_GPIOEnum eGpio, bool bInOut, bool bDefaultState);

        [DllImport(LibName, EntryPoint = "GPIOSetPin")]
        public static extern void GPIOSetPin(DHCOM_GPIOEnum eGpio, bool bState);	            // Sets the GPIO Pin state

        [DllImport(LibName, EntryPoint = "GPIOGetPin")]
        public static extern bool GPIOGetPin(DHCOM_GPIOEnum eGpio);	                            // Returns the GPIO Pin state

        [DllImport(LibName, EntryPoint = "GPIOSetInterrupt")]
        public static extern bool GPIOSetInterrupt(DHCOM_GPIOEnum eGpio, IntPtr hEvent, ulong ulTimeout);

        [DllImport(LibName, EntryPoint = "GPIOClearInterrupt")]
        public static extern bool GPIOClearInterrupt(DHCOM_GPIOEnum eGpio);

        // I2C
        [DllImport(LibName, EntryPoint = "I2COpen")]
        public static extern bool I2COpen(DHCOM_I2CEnum eI2CPort, ulong ulI2CFreq);					// Open the I2C bus

        [DllImport(LibName, EntryPoint = "I2CClose")]                // Close the I2C bus
        public static extern void I2CClose(DHCOM_I2CEnum eI2CPort);

        [DllImport(LibName, EntryPoint = "I2CRead")]
        public static extern bool I2CRead(DHCOM_I2CEnum eI2CPort, DHCOM_I2CFREQMode eI2CFreq, byte cDevId, byte cI2CReg, [MarshalAs(UnmanagedType.FunctionPtr)]byte pOutBuffer);					// Reads from the I2C bus

        [DllImport(LibName, EntryPoint = "I2CWrite")]
        public static extern bool I2CWrite(DHCOM_I2CEnum eI2CPort, DHCOM_I2CFREQMode eI2CFreq, byte cDevId, byte cI2CReg, byte cValue);	// Write to the I2C bus

        [DllImport(LibName, EntryPoint = "I2CMultipleRead")]
        public static extern bool I2CReadMultipleBytes(DHCOM_I2CEnum eI2CPort, DHCOM_I2CFREQMode eI2CFreq, byte cDevId, byte cI2CReg, byte cBytes, [MarshalAs(UnmanagedType.FunctionPtr)]byte pOutBuffer); // Read Multiple Bytes from the I2C bus

        [DllImport(LibName, EntryPoint = "I2CMultipleWrite")]
        public static extern bool I2CWriteMultipleBytes(DHCOM_I2CEnum eI2CPort, DHCOM_I2CFREQMode eI2CFreq, byte cDevId, byte cI2CReg, byte cBytes, [MarshalAs(UnmanagedType.FunctionPtr)]byte pOutBuffer); // Write Multiple Bytes to the I2C bus

        //Physical Address
        [DllImport(LibName, EntryPoint = "ReadPhysicalAddress")]
        public static extern int ReadPhysicalAddress(int iPhyAddr, byte cRegSize);                         		// Reads from Memory (I/O)

        [DllImport(LibName, EntryPoint = "WritePhysicalAddress")]
        public static extern int WritePhysicalAddress(int iPhyAddr, byte cRegSize, int iValue, bool bReadBack);	// Write to Memory (I/O)

        // Backlight
        [DllImport(LibName, EntryPoint = "SetBacklightLevel")]
        public static extern bool SetBacklightLevel(int iLevel);

        [DllImport(LibName, EntryPoint = "SetBacklightSuspend")]
        public static extern bool SetBacklightSuspend(int Timeout, bool bEnable);

        // Touch
        [DllImport(LibName, EntryPoint = "CalibrateTouch")]
        public static extern bool CalibrateTouch();

        [DllImport(LibName, EntryPoint = "ClearTouchCalibrationData")]
        public static extern bool ClearTouchCalibrationData();

        // Reset
        [DllImport(LibName, EntryPoint = "SoftReset")]
        public static extern void SoftReset();

        // Registry
        [DllImport(LibName, EntryPoint = "SaveRegistry")]
        public static extern void SaveRegistry();

        [DllImport(LibName, EntryPoint = "CreateRegistryBackup")]
        public static extern bool CreateRegistryBackup();

        [DllImport(LibName, EntryPoint = "RestoreRegistryBackup")]
        public static extern bool RestoreRegistryBackup();
    }  
#endregion
}
#endregion
